#include "types.h"
#include "io.h"

void _asm_default_int(void);
void _asm_irq_0(void);
void _asm_irq_1(void);

void memcopy(void*, void*, int);

#define IDTBASE  0x800   /* addr. physique ou doit resider la IDT */
#define IDTSIZE  0xFF    /* nombre max. de descripteurs dans la table */
#define INTGATE  0x8E00  /* utilise pour gerer les interruptions */
#define TRAPGATE 0x8F00  /* utilise pour faire des appels systemes */
#define TASKGATE 0x8500  /* utilise pour commuter des taches */

/* descripteur de segment */
struct idtdesc {
    u16 offset0_15;    
    u16 select;
    u16 type;
    u16 offset16_31;    
} __attribute__ ((packed));

/* registre IDTR */
struct idtr {
    u16 limite;
    u32 base;
}  __attribute__ ((packed));

struct idtr kidtr;

/* table de IDT */
struct idtdesc kidt[IDTSIZE];

/* pointeur sur un descripteur libre dans la IDT */
unsigned int kidtptr = 0;


/*
 * 'init_idt_desc' initialise un descripteur de segment situe en idt.
 * 'desc' est l'adresse lineaire du descripteur a initialiser.
 * L'argument 'type' doit avoir pour valeur INTGATE, TRAPGATE
 * ou TASKGATE.
 */
void init_idt_desc(u16 select, u32 offset, u16 type, struct idtdesc* desc) {
    desc->offset0_15 = (offset & 0xffff);
    desc->select = select;
    desc->type = type;
    desc->offset16_31 = (offset & 0xffff0000) >> 16;
    return;
}

/*
 * Cette fonction initialise la IDT apres que le kernel soit charge 
 * en memoire. 
 */
void init_idt(void) {
    struct idtdesc default_desc;
    int i;

    /* initialisation des descripteurs systeme par defaut */
    for(i=0;i<IDTSIZE;i++) {
        init_idt_desc(0x08, (u32) _asm_default_int, INTGATE, &default_desc);
        kidt[kidtptr++] = default_desc;
    }

    init_idt_desc(0x08, (u32) _asm_irq_0, INTGATE, &kidt[32]); /* horloge */
    init_idt_desc(0x08, (u32) _asm_irq_1, INTGATE, &kidt[33]); /* clavier */

    kidtptr = 48;

    /* initialisation de la structure pour IDTR */
    kidtr.limite = IDTSIZE*8;
    kidtr.base = IDTBASE;

    /* recopie de la IDT a son adresse */
    memcopy((char*) kidtr.base, (char*) kidt, kidtr.limite);

    /* chargement du registre IDTR */
    asm("lidtl (kidtr)");
}
